// SpiegelZahl.java                             MM 2013
// Model                                        AZ 2005

/**
 * Problembehandlung:
 * Erzeugen einer symmetrische Zahl durch Spiegelung.
 */
public class SpiegelZahl
{
/* ------------------------------------------------- */
                                            // coreData
/**
 * Spiegelzahl.
 */
  private Zahl zahl;

/**
 * Schrittanzahl, Anzahl der Spiegelungen.
 */
  private int schritte;

/* ------------------------------------------------- */
                                         // Konstruktor
/**
 * Konstruktor, setzt Startwerte.
 */
  public SpiegelZahl()
  {
    reset();
  }

/* ------------------------------------------------- */
                                    // getData-Methoden
/**
 * Lesen der Zahl.
 * @return Zahl als String
 */
  public String getZahl()
  {
    if( zahl == null) return "";
    return zahl.toString();
  }

/**
 * Lesen der Schrittanzahl.
 * @return Anzahl der ausgefuehrten Spiegelungen
 */
  public int getSchritte()
  {
    return schritte;
  }

/* ------------------------------------------------- */
                                    // service-Methoden
/**
 * Setzen einer Zahl,
 * wirft Ausnahme, falls Sting keine Zahl.
 * @param str Zahl als String
 */
  public void setZahl( String str)
    throws Exception
  {
    zahl = new Zahl( str);
    schritte = 0;
  }

/**
 * Zuruecksetzen einer Zahl.
 */
  public void reset()
  {
    zahl = new Zahl();
    schritte = 0;
  }

/**
 * n mal Spiegeln der Zahl,
 * Abbruch, falls Zahl symmetrisch.
 * @return true, falls Zahl symmetrisch
 */
  public boolean spiegeln( int n)
  {
    if( zahl == null) return true;
    if( zahl.istSymmetrisch()) return true;
    for( ; n > 0; n--)
    {
      schritte ++;
      zahl = zahl.addSpiegel();

      if( zahl.istSymmetrisch()) return true;
    }
    return false;
  }

/* ------------------------------------------------- */
/**
 * Darstellung der Spiegelstandes.
 * @return Schrittanzahl und Wert der Zahl als String
 */
  public String toString()
  {
    return getSchritte() + ": " + getZahl();
  }

/* ------------------------------------------------- */
                                              // Regeln
/**
 * Erlaeuterung des Spiegeln von Zahlen.
 * @return Regeln
 */
  public String spielRegeln()
  {
    String str = "Spiegelzahlen\n\n";

    str += "Addiert man eine natuerliche Zahl ";
    str += "mit ihrer Spiegelzahl,\n";
    str += "ist das Ergebnis im allgemeinen ";
    str += "nicht symmetrisch.\n";
    str += "Verfaehrt man mit dem Ergebnis ebenso,\n";
    str += "erhaelt man nach endlich vielen Schritten ";
    str += "eine symmetrische Zahl (Palindrom).\n\n";

    str += "Doch Vorsicht: Das ist nur eine Vermutung!\n";
    str += "Bei verschiedenen Zahlen bricht ";
    str += "der Algorithmus wahrscheinlich niemals ab.";

    return str;
  }

/* ------------------------------------------------- */
                                          // Auswertung
/**
 * Ergebnisauswertung.
 * @return Ergebnis als String
 */
  public String auswertung()
  {
    if( zahl.istSymmetrisch()) return "Fertig!" ;
    return "Problem nicht geloest!";
  }

/* ------------------------------------------------- */
                                            // Programm
/**
 * Test der Klasse SpiegelZahl.
 */
  public static void main( String[] args)
  {
// Problem einrichten
    SpiegelZahl spiegel = new SpiegelZahl();

// Neue Zahl
    try{ spiegel.setZahl( "789");}
    catch( Exception e){}
    System.out.print
    ( spiegel.getZahl() + ": 100x spiegeln ergibt");
// Spiegeln
    if( spiegel.spiegeln( 100))
      System.out.println( " ein Palindrom.");
    else
      System.out.println( " kein Palindrom.");
// Ergebnis
    System.out.println( spiegel);

// Neue Zahl
    System.out.println();
    try{ spiegel.setZahl( "8396501");}
    catch( Exception e){}
    System.out.print
    ( spiegel.getZahl() + ": 100x spiegeln ergibt");
// Spiegeln
    if( spiegel.spiegeln( 100))
      System.out.println( " ein Palindrom");
    else
      System.out.println( " kein Palindrom");
// Ergebnis
    System.out.println( spiegel);

// Neue Zahl
    System.out.println();
    try{ spiegel.setZahl( "456789900345345");}
    catch( Exception e){}
    System.out.print
    ( spiegel.getZahl() + ": 46000x spiegeln ergibt");
// Spiegeln
    if( spiegel.spiegeln( 46000))
      System.out.println( " ein Palindrom");
    else
      System.out.println( " kein Palindrom");
// Ergebnis
    System.out.println( spiegel);

// Beenden
    System.out.println();
    System.out.println( "Programm beendet!");
 }
}

/* ------------------------------------------------- */

// Testwerte:
// 789             -> Schrittzahl  4 :          66066
// 8396501         -> Schrittzahl 17 : 44440011004444
// 456789900345345 -> nach 46000 Schritten
//                    kein Ergebnis!

