// Zahl.java                                    MM 2007
//                                              AZ 2004

import java.util.*;                       // LinkedList

/**
 * Darstellen einer beliebig langen natuerlichen Zahl
 * als LinkedList von Ziffern,
 * Addition des Spiegels einer Zahl und
 * Test auf Symmetrie.
 */

public class Zahl
{
/**
 * Speichern einer beliebig langen natuerlichen Zahl
 * als LinkedList von Ziffern.
 */
  private LinkedList<Ziffer> zahl;

/* ------------------------------------------------- */
/**
 * Konstruktor.
 */
  public Zahl()
  {
    zahl = new LinkedList<Ziffer>();
  }

/**
 * Konstruktor, wandelt String ziffernweise in Zahl um,
 * wirft Ausnahme, falls Sting keine Zahl.
 * @param str Zahl als String
 */
  public Zahl( String str)
  throws Exception
  {
    this();

    int z;
    for( int i = 0; i < str.length(); i++)
    {
      z = Integer.parseInt( str.substring( i, i + 1));
      zahl.add( new Ziffer( z));
    }
  }

/* ------------------------------------------------- */
/**
 * Anfgen einer Ziffer am Anfang einer Zahl.
 * @param z Ziffer
 */
  private void addFirst( Ziffer z)
  {
    zahl.addFirst( z);
  }

/**
 * Symmetrieueberpruefung.
 * @return true bei Symmetrie
 */
  public boolean istSymmetrisch()
  {
                                        // ListIterator
    ListIterator<Ziffer> itNf = zahl.listIterator( 0);
    ListIterator<Ziffer> itVg = zahl.listIterator( zahl.size());

    while( itNf.nextIndex() < itVg.previousIndex())
    {
      if( !( itNf.next()).equals( itVg.previous()))
        return false;
    }
    return true;
  }

/**
 * Addition des Spiegels der Zahl.
 * @return Summe von Zahl und Spiegel der Zahl
 */
  public Zahl addSpiegel()
  {
    Zahl zahlNeu = new Zahl();
    ListIterator<Ziffer> itNf = zahl.listIterator( 0);
    ListIterator<Ziffer> itVg = zahl.listIterator( zahl.size());

    int uebertrag = 0;
    while( itNf.hasNext())
    {
      Ziffer z = new Ziffer( itNf.next());
      uebertrag = z.add( itVg.previous(), uebertrag);
      zahlNeu.addFirst( z);
    }
    if( uebertrag == 1)
      zahlNeu.addFirst( new Ziffer( uebertrag));
    return zahlNeu;
  }

/**
 * Darstellung der Zahl.
 * @return Wert der Zahl als String
 */
  public String toString()
  {
    String str = "";

    Iterator it = zahl.iterator();
    while( it.hasNext()) str += it.next().toString();

    return str;
  }

/* ------------------------------------------------- */
/**
 * Test der Klasse Zahl.
 */
  public static void main( String[] args)
  {
    try
    {
// Objektbildung einer Zahl
     Zahl z = new Zahl( "123");                  // 444
//   Zahl z = new Zahl( "789");                 // 1776
     System.out.println ( "Zahl: " + z);

// Addition mit Spiegel
    z = z.addSpiegel();
    System.out.println ( "Addition mit Spiegel: " + z);

    if( z.istSymmetrisch())
      System.out.println ( "Zahl symmetrisch!");
    }
    catch( Exception e){}                // tue nichts
  }
}
